import click
import os
from pathlib import Path
# from .cmd_responder import responder_cmd_group
from .cmd_app import app_cmd_group

HOME = Path('~/.rxd').expanduser().resolve()


@click.group(name="rxd")
def main():
    pass


@main.command()
@click.option("-u", "--url", required=True, type=str)
@click.option("-a", "--app", required=True, type=str)
def trigger(address: str, app: str):
    import requests
    from .codex import Cipher
    from .messages import RXDRepoUpdatedEvent
    cipher = Cipher(key="")
    message = RXDRepoUpdatedEvent(app_name=app).model_dump_json()
    message_encoded = cipher.encrypt(message).decode('utf-8')
    requests.post(address, json={'payload': message_encoded})


# @main.command()
# @click.option("-w", "--workspace",
#               required=False,
#               type=str,
#               default="workspace",
#               help="Path to workspace directory where your apps will be stored")
# def setup(workspace):

@main.command()
def setup():
    if not HOME.exists():
        print(f"Making {HOME}")
        os.makedirs(HOME)


main.add_command(app_cmd_group)
# main.add_command(responder_cmd_group)

if __name__ == "__main__":
    main()



import click
from .app_manager import AppManager, Application


@click.group(name='app')
def app_cmd_group():
    """Manage applications on serving computers"""
    pass


@app_cmd_group.command()
@click.argument('name')
@click.argument('repo')
def add(name, repo):
    """
    Add app to with <name> and <repo> to rxd
    """
    print(f"adding {name} {repo}")
    app = Application(name, repo)
    if app.exists():
        print(f"Application with name '{app.name}' already exists")
    else:
        app.save()


@app_cmd_group.command()
@click.argument('name')
def fetch(name):
    """
    Pull latest commit from app's repository
    """
    app = Application.load(name)
    if not app.exists():
        print(f"Application with name '{app.name}' does not exists")
    else:
        app.fetch()


@app_cmd_group.command()
@click.argument('name')
def build(name):
    """
    Build app by cd-ing into app workspace and running .deploy/build
    """
    app = Application.load(name)
    if not app.exists():
        print(f"Application with name '{app.name}' does not exists")
    else:
        app.build()


@app_cmd_group.command()
@click.argument('name')
def install(name):
    """
    Install a systemd service file to call .deploy/run upon system
    startup
    """
    app = Application.load(name)
    if not app.exists():
        print(f"Application with name '{app.name}' does not exists")
    else:
        app.daemonize()


@app_cmd_group.command()
@click.argument('name')
def run(name):
    """
    CD into app workspace and run it using .deploy/run
    """
    app = Application.load(name)
    if not app.exists():
        print(f"Application with name '{app.name}' does not exists")
    else:
        app.run()


@app_cmd_group.command()
def list():
    """
    List all apps currently installed
    """
    from tabulate import tabulate
    am = AppManager()
    table = []
    for app in am.list():
        status = app.status()
        # currently installed in systemd
        if status:
            table.append([app.name,
                          app.repo,
                          status['is_running'],
                          status['state'],
                          status['pid'],
                          status['memory_mb']
                          ])
        else:
            table.append([app.name,
                          app.repo,
                          None,
                          None,
                          None,
                          None])
    print(tabulate(table, headers=['Name',
                                   'Repo',
                                   'Running',
                                   'State',
                                   'PID',
                                   'Mem MB'],
                   tablefmt='mixed_outline'))


@app_cmd_group.command()
@click.argument('name')
def status(name):
    """
    Get App's daemon status
    """
    app = Application.load(name)
    if not app.exists():
        print(f"Application with name '{app.name}' does not exists")
        return
    app.status(output=True)


@app_cmd_group.command()
@click.argument('name')
def stop(name):
    """
    Stop App's daemon
    """
    app = Application.load(name)
    if not app.exists():
        print(f"Application with name '{app.name}' does not exists")
        return

    app.stop()


@app_cmd_group.command()
@click.argument('name')
def start(name):
    """
    Start App's daemon
    """
    app = Application.load(name)
    if not app.exists():
        print(f"Application with name '{app.name}' does not exists")
        return

    app.start()


@app_cmd_group.command()
@click.argument('name')
def enable(name):
    """
    Enable App's daemon
    """
    app = Application.load(name)
    if not app.exists():
        print(f"Application with name '{app.name}' does not exists")
        return

    app.enable()


@app_cmd_group.command()
@click.argument('name')
def disable(name):
    """
    Disable App's daemon
    """
    app = Application.load(name)
    if not app.exists():
        print(f"Application with name '{app.name}' does not exists")
        return
    app.disable()


@app_cmd_group.command()
@click.argument('name')
def restart(name):
    """
    Restart App's daemon
    """
    app = Application.load(name)
    if not app.exists():
        print(f"Application with name '{app.name}' does not exists")
        return
    app.restart()


@app_cmd_group.command()
@click.argument('name')
@click.option('-f', '--follow',
              is_flag=True,
              show_default=True,
              default=False)
def logs(name, follow=False):
    """
    Get App's daemon's log
    """
    app = Application.load(name)
    if not app.exists():
        print(f"Application with name '{app.name}' does not exists")
        return

    app.logs(follow)


if __name__ == '__main__':
    app_cmd_group()



#!/usr/bin/env python

from setuptools import setup
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()


setup(name='rxd',
      version='0.0.18',
      description='A reactive application manager',
      long_description=long_description,
      long_description_content_type='text/markdown',
      author='Huy Nguyen',
      author_email='121183+huyng@users.noreply.github.com',
      packages=['rxd'],
      entry_points={
          'console_scripts': [
              'rxd = rxd.cmd:main',
          ],
      },
      install_requires=["Flask", "Click", "tabulate"],
      zip_safe=False,
      url="https://github.com/huyng/rxd")

# to distribute run:
# python setup.py register sdist upload
# python -m twine  upload dist
